@@ -177,22 +177,69 @@ def import_data(type, input_path, output_path, input_format):
                help="Display basic information about a QIIME 2 Artifact or "
                     "Visualization, including its UUID and type.",
                cls=ToolCommand)
-@click.argument('path', type=click.Path(exists=True, file_okay=True,
-                                        dir_okay=False, readable=True),
-                metavar=_COMBO_METAVAR)
-def peek(path):
+@click.argument('paths', nargs=-1, required=True,
+                type=click.Path(exists=True, file_okay=True, dir_okay=False,
+                                readable=True), metavar=_COMBO_METAVAR)
+@click.option('--tsv/--no-tsv', default=False,
+              help='Print as machine-readable tab separated values.')
+def peek(paths, tsv):
     import qiime2.sdk
     from q2cli.core.config import CONFIG
 
-    metadata = qiime2.sdk.Result.peek(path)
+    metadatas = {os.path.basename(path):
+                 qiime2.sdk.Result.peek(path) for path in paths}
 
-    click.echo(CONFIG.cfg_style('type', "UUID")+":        ", nl=False)
-    click.echo(metadata.uuid)
-    click.echo(CONFIG.cfg_style('type', "Type")+":        ", nl=False)
-    click.echo(metadata.type)
-    if metadata.format is not None:
-        click.echo(CONFIG.cfg_style('type', "Data format")+": ", nl=False)
-        click.echo(metadata.format)
+    if tsv:
+        click.echo("Filename\tType\tUUID\tData Format")
+        for path, m in metadatas.items():
+            click.echo(f"{path}\t{m.type}\t{m.uuid}\t{m.format}")
+
+    elif len(metadatas) == 1:
+        metadata = metadatas[os.path.basename(paths[0])]
+        click.echo(CONFIG.cfg_style('type', "UUID")+":        ", nl=False)
+        click.echo(metadata.uuid)
+        click.echo(CONFIG.cfg_style('type', "Type")+":        ", nl=False)
+        click.echo(metadata.type)
+        if metadata.format is not None:
+            click.echo(CONFIG.cfg_style('type', "Data format")+": ", nl=False)
+            click.echo(metadata.format)
+
+    else:
+        COLUMN_FILENAME = "Filename"
+        COLUMN_TYPE = "Type"
+        COLUMN_UUID = "UUID"
+        COLUMN_DATA_FORMAT = "Data Format"
+
+        filename_width = max([len(p) for p in paths]
+                             + [len(COLUMN_FILENAME)])
+        type_width = max([len(i.type) for i in metadatas.values()]
+                         + [len(COLUMN_TYPE)])
+        uuid_width = max([len(i.uuid) for i in metadatas.values()]
+                         + [len(COLUMN_UUID)])
+        data_format_width = \
+            max([len(i.format) if i.format is not None else 0
+                 for i in metadatas.values()] + [len(COLUMN_DATA_FORMAT)])
+
+        padding = 2
+        format_string = f"{{f:<{filename_width + padding}}} " + \
+                        f"{{t:<{type_width + padding}}} " + \
+                        f"{{u:<{uuid_width + padding}}} " + \
+                        f"{{d:<{data_format_width + padding}}}"
+
+        click.secho(
+            format_string.format(
+                f=COLUMN_FILENAME,
+                t=COLUMN_TYPE,
+                u=COLUMN_UUID,
+                d=COLUMN_DATA_FORMAT),
+            bold=True, fg="green")
+        for path, m in metadatas.items():
+            click.echo(
+                format_string.format(
+                    f=path,
+                    t=m.type,
+                    u=m.uuid,
+                    d=(m.format if m.format is not None else 'N/A')))
 
 
 _COLUMN_TYPES = ['categorical', 'numeric']
@@ -324,8 +371,7 @@ def cast_metadata(paths, cast, output_file, ignore_extra,
 @click.option('--tsv/--no-tsv', default=False,
               help='Print as machine-readable TSV instead of text.')
 @click.argument('paths', nargs=-1, required=True, metavar='METADATA...',
-                type=click.Path(exists=True, file_okay=True, dir_okay=False,
-                                readable=True))
+                type=click.Path(file_okay=True, dir_okay=False, readable=True))
 @q2cli.util.pretty_failure(traceback=None)
 def inspect_metadata(paths, tsv, failure):
     metadata = _merge_metadata(paths)
@@ -373,30 +419,8 @@ def inspect_metadata(paths, tsv, failure):
         click.echo(metadata.column_count)
 
 
-def _load_metadata(path):
-    import qiime2
-    import qiime2.sdk
-
-    # TODO: clean up duplication between this and the metadata handlers.
-    try:
-        artifact = qiime2.sdk.Result.load(path)
-    except Exception:
-        metadata = qiime2.Metadata.load(path)
-    else:
-        if isinstance(artifact, qiime2.Visualization):
-            raise Exception("Visualizations cannot be viewed as QIIME 2"
-                            " metadata:\n%r" % path)
-        elif artifact.has_metadata():
-            metadata = artifact.view(qiime2.Metadata)
-        else:
-            raise Exception("Artifacts with type %r cannot be viewed as"
-                            " QIIME 2 metadata:\n%r" % (artifact.type, path))
-
-    return metadata
-
-
 def _merge_metadata(paths):
-    m = [_load_metadata(p) for p in paths]
+    m = [q2cli.util.load_metadata(p) for p in paths]
     metadata = m[0]
     if m[1:]:
         metadata = metadata.merge(*m[1:])
@@ -410,14 +434,15 @@ def _merge_metadata(paths):
                     "used after the command exits, use 'qiime tools extract'.",
                cls=ToolCommand)
 @click.argument('visualization-path', metavar='VISUALIZATION',
-                type=click.Path(exists=True, file_okay=True, dir_okay=False,
-                                readable=True))
+                type=click.Path(file_okay=True, dir_okay=False, readable=True))
 @click.option('--index-extension', required=False, default='html',
               help='The extension of the index file that should be opened. '
                    '[default: html]')
 def view(visualization_path, index_extension):
     # Guard headless envs from having to import anything large
     import sys
+    from qiime2 import Visualization
+    from q2cli.util import _load_input
     from q2cli.core.config import CONFIG
     if not os.getenv("DISPLAY") and sys.platform != "darwin":
         raise click.UsageError(
@@ -426,16 +451,11 @@ def view(visualization_path, index_extension):
             'https://view.qiime2.org, or move the Visualization to an '
             'environment with a display and view it with `qiime tools view`.')
 
-    import zipfile
-    import qiime2.sdk
-
     if index_extension.startswith('.'):
         index_extension = index_extension[1:]
-    try:
-        visualization = qiime2.sdk.Visualization.load(visualization_path)
-    # TODO: currently a KeyError is raised if a zipped file that is not a
-    # QIIME 2 result is passed. This should be handled better by the framework.
-    except (zipfile.BadZipFile, KeyError, TypeError):
+
+    visualization = _load_input(visualization_path, view=True)[0]
+    if not isinstance(visualization, Visualization):
         raise click.BadParameter(
             '%s is not a QIIME 2 Visualization. Only QIIME 2 Visualizations '
             'can be viewed.' % visualization_path)
@@ -580,3 +600,199 @@ def citations(path):
         click.echo(CONFIG.cfg_style('problem', 'No citations found.'),
                    err=True)
         ctx.exit(1)
+
+
+@tools.command(name='cache-create',
+               short_help='Create an empty cache at the given location.',
+               help='Create an empty cache at the given location.',
+               cls=ToolCommand)
+@click.option('--cache', required=True,
+              type=click.Path(exists=False, readable=True),
+              help='Path to a nonexistent directory to be created as a cache.')
+def cache_create(cache):
+    from qiime2.core.cache import Cache
+    from q2cli.core.config import CONFIG
+
+    try:
+        Cache(cache)
+    except Exception as e:
+        header = "There was a problem creating a cache at '%s':" % cache
+        q2cli.util.exit_with_error(e, header=header, traceback=None)
+
+    success = "Created cache at '%s'" % cache
+    click.echo(CONFIG.cfg_style('success', success))
+
+
+@tools.command(name='cache-remove',
+               short_help='Removes a given key from a cache.',
+               help='Removes a given key from a cache then runs garbage '
+                    'collection on the cache.',
+               cls=ToolCommand)
+@click.option('--cache', required=True,
+              type=click.Path(exists=True, file_okay=False, dir_okay=True,
+                              readable=True),
+              help='Path to an existing cache to remove the key from.')
+@click.option('--key', required=True,
+              help='The key to remove from the cache.')
+def cache_remove(cache, key):
+    from qiime2.core.cache import Cache
+    from q2cli.core.config import CONFIG
+
+    try:
+        _cache = Cache(cache)
+        _cache.remove(key)
+    except Exception as e:
+        header = "There was a problem removing the key '%s' from the " \
+                 "cache '%s':" % (key, cache)
+        q2cli.util.exit_with_error(e, header=header, traceback=None)
+
+    success = "Removed key '%s' from cache '%s'" % (key, cache)
+    click.echo(CONFIG.cfg_style('success', success))
+
+
+@tools.command(name='cache-garbage-collection',
+               short_help='Runs garbage collection on the cache at the '
+                          'specified location.',
+               help='Runs garbage collection on the cache at the specified '
+                    'location if the specified location is a cache.',
+               cls=ToolCommand)
+@click.option('--cache', required=True,
+              type=click.Path(exists=True, file_okay=False, dir_okay=True,
+                              readable=True),
+              help='Path to an existing cache to run garbage collection on.')
+def cache_garbage_collection(cache):
+    from qiime2.core.cache import Cache
+    from q2cli.core.config import CONFIG
+
+    try:
+        _cache = Cache(cache)
+        _cache.garbage_collection()
+    except Exception as e:
+        header = "There was a problem running garbage collection on the " \
+            "cache at '%s':" % cache
+        q2cli.util.exit_with_error(e, header=header, traceback=None)
+
+    success = "Ran garbage collection on cache at '%s'" % cache
+    click.echo(CONFIG.cfg_style('success', success))
+
+
+@tools.command(name='cache-store',
+               short_help='Stores a .qza in the cache under a key.',
+               help='Stores a .qza in the cache under a key.',
+               cls=ToolCommand)
+@click.option('--cache', required=True,
+              type=click.Path(exists=True, file_okay=False, dir_okay=True,
+                              readable=True),
+              help='Path to an existing cache to save into.')
+@click.option('--artifact-path', required=True,
+              type=click.Path(exists=True, file_okay=True, dir_okay=False,
+                              readable=True),
+              help='Path to a .qza to save into the cache.')
+@click.option('--key', required=True,
+              help='The key to save the artifact under (must be a valid '
+                   'Python identifier).')
+def cache_store(cache, artifact_path, key):
+    from qiime2.sdk.result import Result
+    from qiime2.core.cache import Cache
+    from q2cli.core.config import CONFIG
+
+    try:
+        artifact = Result.load(artifact_path)
+        _cache = Cache(cache)
+        _cache.save(artifact, key)
+    except Exception as e:
+        header = "There was a problem saving the artifact '%s' to the cache " \
+                 "'%s' under the key '%s':" % (artifact_path, cache, key)
+        q2cli.util.exit_with_error(e, header=header, traceback=None)
+
+    success = "Saved the artifact '%s' to the cache '%s' under the key " \
+        "'%s'" % (artifact_path, cache, key)
+    click.echo(CONFIG.cfg_style('success', success))
+
+
+@tools.command(name='cache-fetch',
+               short_help='Fetches an artifact out of a cache into a .qza.',
+               help='Fetches the artifact saved to the specified cache under '
+                    'the specified key into a .qza at the specified location.',
+               cls=ToolCommand)
+@click.option('--cache', required=True,
+              type=click.Path(exists=True, file_okay=False, dir_okay=True,
+                              readable=True),
+              help='Path to an existing cache to load from.')
+@click.option('--key', required=True,
+              help='The key to the artifact being loaded.')
+@click.option('--output-path', required=True,
+              type=click.Path(exists=False, readable=True),
+              help='Path to put the .qza we are loading the artifact into.')
+def cache_fetch(cache, key, output_path):
+    from qiime2.core.cache import Cache
+    from q2cli.core.config import CONFIG
+
+    try:
+        _cache = Cache(cache)
+        artifact = _cache.load(key)
+        artifact.save(output_path)
+    except Exception as e:
+        header = "There was a problem loading the artifact with the key " \
+                 "'%s' from the cache '%s' and saving it to the file '%s':" % \
+                 key, cache, output_path
+        q2cli.util.exit_with_error(e, header=header, traceback=None)
+
+    success = "Loaded artifact with the key '%s' from the cache '%s' and " \
+        "saved it to the file '%s'" % (key, cache, output_path)
+    click.echo(CONFIG.cfg_style('success', success))
+
+
+@tools.command(name='cache-status',
+               short_help='Checks the status of the cache.',
+               help='Lists all keys in the given cache. Peeks artifacts '
+                    'pointed to by keys to data and lists the number of '
+                    'artifacts in the pool for keys to pools.',
+               cls=ToolCommand)
+@click.option('--cache', required=True,
+              type=click.Path(exists=True, file_okay=False, dir_okay=True,
+                              readable=True),
+              help='Path to an existing cache to check the status of.')
+def cache_status(cache):
+    from qiime2.core.cache import Cache
+    from qiime2.sdk.result import Result
+
+    from q2cli.core.config import CONFIG
+
+    data_output = []
+    pool_output = []
+    try:
+        _cache = Cache(cache)
+        with _cache.lock:
+            for key in _cache.get_keys():
+                key_values = _cache.read_key(key)
+
+                if (data := key_values['data']) is not None:
+                    data_output.append(
+                        'data: %s -> %s' %
+                        (key, str(Result.peek(_cache.data / data))))
+                elif (pool := key_values['pool']) is not None:
+                    pool_output.append(
+                        'pool: %s -> size = %s' %
+                        (key, str(len(os.listdir(_cache.pools / pool)))))
+    except Exception as e:
+        header = "There was a problem getting the status of the cache at " \
+                 "path '%s':" % cache
+        q2cli.util.exit_with_error(e, header=header, traceback=None)
+
+    if not data_output:
+        data_output = 'No data keys in cache'
+    else:
+        data_output = '\n'.join(data_output)
+        data_output = 'Data keys in cache:\n' + data_output
+
+    if not pool_output:
+        pool_output = 'No pool keys in cache'
+    else:
+        pool_output = '\n'.join(pool_output)
+        pool_output = 'Pool keys in cache:\n' + pool_output
+
+    output = data_output + '\n\n' + pool_output
+    success = "Status of the cache at the path '%s':\n\n%s" % \
+        (cache, output)
+    click.echo(CONFIG.cfg_style('success', success))
