@@ -7,6 +7,7 @@
 # ----------------------------------------------------------------------------
 
 import os
+import gc
 import shutil
 import unittest
 import tempfile
@@ -15,8 +16,11 @@ from click.testing import CliRunner
 from qiime2 import Artifact
 from qiime2.core.testing.util import get_dummy_plugin
 from qiime2.metadata.base import SUPPORTED_COLUMN_TYPES
+from qiime2.core.cache import Cache
+from qiime2.sdk.result import Result
 
-from q2cli.builtin.tools import tools, _load_metadata
+from q2cli.util import load_metadata
+from q2cli.builtin.tools import tools
 from q2cli.commands import RootCommand
 
 
@@ -102,7 +106,7 @@ class TestCastMetadata(unittest.TestCase):
                     '--ignore-extra', '--output-file', self.output_file])
 
         self.assertEqual(result.exit_code, 0)
-        casted_metadata = _load_metadata(self.output_file)
+        casted_metadata = load_metadata(self.output_file)
         self.assertNotIn('extra', casted_metadata.columns.keys())
 
     def test_complete_successful_run(self):
@@ -111,10 +115,10 @@ class TestCastMetadata(unittest.TestCase):
                     'numbers:categorical', '--output-file', self.output_file])
 
         self.assertEqual(result.exit_code, 0)
-        input_metadata = _load_metadata(self.metadata_file)
+        input_metadata = load_metadata(self.metadata_file)
         self.assertEqual('numeric', input_metadata.columns['numbers'].type)
 
-        casted_metadata = _load_metadata(self.output_file)
+        casted_metadata = load_metadata(self.output_file)
         self.assertEqual('categorical',
                          casted_metadata.columns['numbers'].type)
 
@@ -445,5 +449,280 @@ class TestExportToFileFormat(TestInspectMetadata):
         self.assertEqual(success, result.output)
 
 
+class TestCacheTools(unittest.TestCase):
+    def setUp(self):
+        get_dummy_plugin()
+
+        self.runner = CliRunner()
+        self.plugin_command = RootCommand().get_command(
+            ctx=None, name='dummy-plugin')
+        self.tempdir = \
+            tempfile.TemporaryDirectory(prefix='qiime2-q2cli-test-temp-')
+
+        self.art1 = Artifact.import_data('IntSequence1', [0, 1, 2])
+        self.art2 = Artifact.import_data('IntSequence1', [3, 4, 5])
+        self.art3 = Artifact.import_data('IntSequence1', [6, 7, 8])
+        self.art4 = Artifact.import_data('IntSequence2', [9, 10, 11])
+        self.cache = Cache(os.path.join(self.tempdir.name, 'new_cache'))
+
+    def tearDown(self):
+        self.tempdir.cleanup()
+
+    def test_cache_create(self):
+        cache_path = os.path.join(self.tempdir.name, 'created_cache')
+
+        result = self.runner.invoke(
+            tools, ['cache-create', '--cache', cache_path])
+
+        success = "Created cache at '%s'\n" % cache_path
+        self.assertEqual(success, result.output)
+        self.assertTrue(Cache.is_cache(cache_path))
+
+    def test_cache_remove(self):
+        self.cache.save(self.art1, 'key')
+        self.assertTrue('key' in self.cache.get_keys())
+
+        result = self.runner.invoke(
+            tools,
+            ['cache-remove', '--cache', str(self.cache.path), '--key', 'key'])
+
+        success = "Removed key 'key' from cache '%s'\n" % self.cache.path
+        self.assertEqual(success, result.output)
+        self.assertFalse('key' in self.cache.get_keys())
+
+    def test_cache_garbage_collection(self):
+        # Data referenced directly by key
+        self.cache.save(self.art1, 'foo')
+        # Data referenced by pool that is referenced by key
+        pool = self.cache.create_pool(['bar'])
+        pool.save(self.art2)
+        # We will be manually deleting the keys that back these two
+        self.cache.save(self.art3, 'baz')
+        pool = self.cache.create_pool(['qux'])
+        pool.save(self.art4)
+
+        # What we expect to see before and after gc
+        expected_pre_gc_contents = \
+            set(('./VERSION', 'keys/foo', 'keys/bar',
+                 'keys/baz', 'keys/qux',
+                 f'pools/bar/{self.art2.uuid}',
+                 f'pools/qux/{self.art4.uuid}',
+                 f'data/{self.art1.uuid}', f'data/{self.art2.uuid}',
+                 f'data/{self.art3.uuid}', f'data/{self.art4.uuid}'))
+
+        expected_post_gc_contents = \
+            set(('./VERSION', 'keys/foo', 'keys/bar',
+                 f'pools/bar/{self.art2.uuid}',
+                 f'data/{self.art1.uuid}', f'data/{self.art2.uuid}'))
+
+        # Assert cache looks how we want pre gc
+        pre_gc_contents = _get_cache_contents(self.cache)
+        self.assertEqual(expected_pre_gc_contents, pre_gc_contents)
+
+        # Delete keys
+        self.cache.remove(self.cache.keys / 'baz')
+        self.cache.remove(self.cache.keys / 'qux')
+
+        # Make sure Python's garbage collector gets the process pool symlinks
+        # to the artifact that was keyed on baz and the one in the qux pool
+        gc.collect()
+        result = self.runner.invoke(
+            tools,
+            ['cache-garbage-collection', '--cache', str(self.cache.path)])
+
+        success = "Ran garbage collection on cache at '%s'\n" % self.cache.path
+        self.assertEqual(success, result.output)
+
+        # Assert cache looks how we want post gc
+        post_gc_contents = _get_cache_contents(self.cache)
+        self.assertEqual(expected_post_gc_contents, post_gc_contents)
+
+    def test_cache_store(self):
+        artifact = os.path.join(self.tempdir.name, 'artifact.qza')
+        self.art1.save(artifact)
+
+        result = self.runner.invoke(
+            tools, ['cache-store', '--cache', str(self.cache.path),
+                    '--artifact-path', artifact, '--key', 'key'])
+
+        success = "Saved the artifact '%s' to the cache '%s' under the key " \
+            "'key'\n" % (artifact, self.cache.path)
+        self.assertEqual(success, result.output)
+
+    def test_cache_fetch(self):
+        artifact = os.path.join(self.tempdir.name, 'artifact.qza')
+        self.cache.save(self.art1, 'key')
+
+        result = self.runner.invoke(
+            tools, ['cache-fetch', '--cache', str(self.cache.path),
+                    '--key', 'key', '--output-path', artifact])
+
+        success = "Loaded artifact with the key 'key' from the cache '%s' " \
+            "and saved it to the file '%s'\n" % (self.cache.path, artifact)
+        self.assertEqual(success, result.output)
+
+    def test_cache_roundtrip(self):
+        in_artifact = os.path.join(self.tempdir.name, 'in_artifact.qza')
+        out_artifact = os.path.join(self.tempdir.name, 'out_artifact.qza')
+
+        self.art1.save(in_artifact)
+
+        result = self.runner.invoke(
+            tools, ['cache-store', '--cache', str(self.cache.path),
+                    '--artifact-path', in_artifact, '--key', 'key'])
+
+        success = "Saved the artifact '%s' to the cache '%s' under the key " \
+            "'key'\n" % (in_artifact, self.cache.path)
+        self.assertEqual(success, result.output)
+
+        result = self.runner.invoke(
+            tools, ['cache-fetch', '--cache', str(self.cache.path),
+                    '--key', 'key', '--output-path', out_artifact])
+
+        success = "Loaded artifact with the key 'key' from the cache '%s' " \
+            "and saved it to the file '%s'\n" % (self.cache.path, out_artifact)
+        self.assertEqual(success, result.output)
+
+        artifact = Artifact.load(out_artifact)
+        self.assertEqual([0, 1, 2], artifact.view(list))
+
+    def test_cache_status(self):
+        success_template = \
+            "Status of the cache at the path '%s':\n\n%s\n\n%s\n"
+
+        # Empty cache
+        result = self.runner.invoke(
+            tools, ['cache-status', '--cache', str(self.cache.path)])
+        success = \
+            success_template % (str(self.cache.path), 'No data keys in cache',
+                                'No pool keys in cache')
+        self.assertEqual(success, result.output)
+
+        # Cache with only data
+        in_artifact = os.path.join(self.tempdir.name, 'in_artifact.qza')
+        self.art1.save(in_artifact)
+        self.runner.invoke(
+            tools, ['cache-store', '--cache', str(self.cache.path),
+                    '--artifact-path', in_artifact, '--key', 'key'])
+
+        result = self.runner.invoke(
+            tools, ['cache-status', '--cache', str(self.cache.path)])
+        data_output = 'Data keys in cache:\ndata: key -> %s' % \
+            str(Result.peek(self.cache.data / str(self.art1.uuid)))
+        success = \
+            success_template % (str(self.cache.path), data_output,
+                                'No pool keys in cache')
+        self.assertEqual(success, result.output)
+
+        # Cache with data and pool
+        pool = self.cache.create_pool(keys=['pool_key'])
+        pool.save(self.art2)
+
+        result = self.runner.invoke(
+            tools, ['cache-status', '--cache', str(self.cache.path)])
+        pool_output = 'Pool keys in cache:\npool: pool_key -> size = 1'
+        success = \
+            success_template % (str(self.cache.path), data_output,
+                                pool_output)
+        self.assertEqual(success, result.output)
+
+
+def _get_cache_contents(cache):
+    """Gets contents of cache not including contents of the artifacts
+    themselves relative to the root of the cache
+    """
+    cache_contents = set()
+
+    rel_keys = os.path.relpath(cache.keys, cache.path)
+    rel_data = os.path.relpath(cache.data, cache.path)
+    rel_pools = os.path.relpath(cache.pools, cache.path)
+    rel_cache = os.path.relpath(cache.path, cache.path)
+
+    for key in os.listdir(cache.keys):
+        cache_contents.add(os.path.join(rel_keys, key))
+
+    for art in os.listdir(cache.data):
+        cache_contents.add(os.path.join(rel_data, art))
+
+    for pool in os.listdir(cache.pools):
+        for link in os.listdir(os.path.join(cache.pools, pool)):
+            cache_contents.add(os.path.join(rel_pools, pool, link))
+
+    for elem in os.listdir(cache.path):
+        if os.path.isfile(os.path.join(cache.path, elem)):
+            cache_contents.add(os.path.join(rel_cache, elem))
+
+    return cache_contents
+
+
+class TestPeek(unittest.TestCase):
+    def setUp(self):
+        self.runner = CliRunner()
+        self.tempdir = tempfile.mkdtemp(prefix='qiime2-q2cli-test-temp-')
+
+        # create artifact
+        self.artifact = os.path.join(self.tempdir, 'artifact.qza')
+        Artifact.import_data(
+            'Mapping', {'foo': 'bar'}).save(self.artifact)
+
+        # create visualization
+        qiime_cli = RootCommand()
+        command = qiime_cli.get_command(ctx=None, name='dummy-plugin')
+        self.viz = os.path.join(self.tempdir, 'viz.qzv')
+
+        self.ints = os.path.join(self.tempdir, 'ints.qza')
+        ints = Artifact.import_data(
+            'IntSequence1', [0, 42, 43], list)
+        ints.save(self.ints)
+
+        self.runner.invoke(
+            command, ['most-common-viz', '--i-ints', self.ints,
+                      '--o-visualization', self.viz, '--verbose'])
+
+    def tearDown(self):
+        shutil.rmtree(self.tempdir)
+
+    def test_single_artifact(self):
+        result = self.runner.invoke(tools, ['peek', self.artifact])
+        self.assertEqual(result.exit_code, 0)
+        self.assertIn("UUID:", result.output)
+        self.assertIn("Type:", result.output)
+        self.assertIn("Data format:", result.output)
+        self.assertEqual(result.output.count('\n'), 3)
+
+    def test_single_visualization(self):
+        result = self.runner.invoke(tools, ['peek', self.viz])
+        self.assertEqual(result.exit_code, 0)
+        self.assertIn("UUID:", result.output)
+        self.assertIn("Type:", result.output)
+        self.assertNotIn("Data format:", result.output)
+        self.assertEqual(result.output.count('\n'), 2)
+
+    def test_artifact_and_visualization(self):
+        result = self.runner.invoke(tools, ['peek', self.artifact, self.viz])
+        self.assertEqual(result.exit_code, 0)
+        self.assertIn("UUID", result.output)
+        self.assertIn("Type", result.output)
+        self.assertIn("Data Format", result.output)
+        self.assertIn("N/A", result.output)
+        self.assertEqual(result.output.count('\n'), 3)
+
+    def test_single_file_tsv(self):
+        result = self.runner.invoke(tools, ['peek', '--tsv', self.artifact])
+        self.assertIn("Filename\tType\tUUID\tData Format\n", result.output)
+        self.assertIn("artifact.qza", result.output)
+        self.assertEqual(result.output.count('\t'), 6)
+        self.assertEqual(result.output.count('\n'), 2)
+
+    def test_multiple_file_tsv(self):
+        result = self.runner.invoke(tools, ['peek', '--tsv', self.artifact,
+                                            self.viz])
+        self.assertIn("Filename\tType\tUUID\tData Format\n", result.output)
+        self.assertIn("artifact.qza", result.output)
+        self.assertIn("viz.qzv", result.output)
+        self.assertEqual(result.output.count('\t'), 9)
+        self.assertEqual(result.output.count('\n'), 3)
+
+
 if __name__ == "__main__":
     unittest.main()
